@@ -1,25 +1,30 @@
-from GramAddict.core.utils import get_value, random_sleep
 import logging
+import os
+from datetime import timedelta
 from functools import partial
 from os import path
+
+from atomicwrites import atomic_write
 from colorama import Fore
+
 from GramAddict.core.device_facade import Direction, Timeout
-from GramAddict.core.storage import FollowingStatus
 from GramAddict.core.navigation import (
+    nav_to_blogger,
     nav_to_feed,
     nav_to_hashtag_or_place,
-    nav_to_blogger,
     nav_to_post_likers,
 )
 from GramAddict.core.resources import ClassName
+from GramAddict.core.storage import FollowingStatus
+from GramAddict.core.utils import get_value, random_choice, random_sleep
 from GramAddict.core.views import (
     FollowingView,
-    PostsViewList,
+    LikeMode,
     OpenedPostView,
+    Owner,
+    PostsViewList,
     ProfileView,
     SwipeTo,
-    Owner,
-    LikeMode,
     TabBarView,
     UniversalActions,
     case_insensitive_re,
@@ -38,13 +43,11 @@ def interact(
     current_job,
     on_interaction,
 ):
+    can_follow = False
     if is_follow_limit_reached is not None:
-        can_follow = not is_follow_limit_reached() and (
-            storage.get_following_status(username) == FollowingStatus.NONE
-            or storage.get_following_status(username) == FollowingStatus.NOT_IN_LIST
-        )
-    else:
-        can_follow = False
+        can_follow = not is_follow_limit_reached() and storage.get_following_status(
+            username
+        ) in [FollowingStatus.NONE, FollowingStatus.NOT_IN_LIST]
 
     (
         interaction_succeed,
@@ -72,25 +75,15 @@ def interact(
         commented=number_of_comments,
         pm_sent=pm_sent,
     )
-    can_continue = on_interaction(
+    return on_interaction(
         succeed=interaction_succeed,
         followed=followed,
         scraped=scraped,
     )
-    return can_continue
-
-
-def random_choice(interact_percentage):
-    from random import randint
-
-    random_number = randint(1, 100)
-    if interact_percentage > random_number:
-        return True
-    else:
-        return False
 
 
 def handle_blogger(
+    self,
     device,
     session_state,
     blogger,
@@ -101,12 +94,25 @@ def handle_blogger(
     interaction,
     is_follow_limit_reached,
 ):
-    nav_to_blogger(device, blogger, session_state.my_username)
+    if not nav_to_blogger(device, blogger, session_state.my_username):
+        return
+    can_interact = False
     if storage.is_user_in_blacklist(blogger):
         logger.info(f"@{blogger} is in blacklist. Skip.")
-    elif storage.check_user_was_interacted(blogger):
-        logger.info(f"@{blogger}: already interacted. Skip.")
     else:
+        interacted, interacted_when = storage.check_user_was_interacted(blogger)
+        if interacted:
+            interact_after = timedelta(hours=float(self.args.can_reinteract_after))
+            can_reinteract = storage.can_be_reinteract(interacted_when, interact_after)
+            logger.info(
+                f"@{blogger}: already interacted on {interacted_when:%Y/%m/%d %H:%M:%S}. {'Interacting again now' if can_reinteract else 'Skip'}."
+            )
+            if can_reinteract:
+                can_interact = True
+        else:
+            can_interact = True
+
+    if can_interact:
         logger.info(
             f"@{blogger}: interact",
             extra={"color": f"{Fore.YELLOW}"},
@@ -127,7 +133,7 @@ def handle_blogger(
 def handle_blogger_from_file(
     self,
     device,
-    current_filename,
+    parameter_passed,
     current_job,
     storage,
     on_interaction,
@@ -137,73 +143,133 @@ def handle_blogger_from_file(
     need_to_refresh = True
     on_following_list = False
     limit_reached = False
-    if path.isfile(current_filename):
-        with open(current_filename, "r") as f:
-            for line in f:
-                username = line.strip()
-                if username != "":
-                    if current_job == "unfollow-from-file":
-                        unfollowed = do_unfollow_from_list(
-                            device, username, on_following_list
-                        )
-                        on_following_list = True
-                        if unfollowed:
-                            storage.add_interacted_user(
-                                username, self.session_state.id, unfollowed=True
-                            )
-                            self.session_state.totalUnfollowed += 1
-                            limit_reached = self.session_state.check_limit(
-                                self.args, limit_type=self.session_state.Limit.UNFOLLOWS
-                            )
-                        if limit_reached:
-                            logger.info("Unfollows limit reached.")
-                            break
-                    else:
-                        if storage.is_user_in_blacklist(username):
-                            logger.info(f"@{username} is in blacklist. Skip.")
-                            continue
-                        elif storage.check_user_was_interacted(username):
-                            logger.info(f"@{username}: already interacted. Skip.")
-                            continue
-                        if need_to_refresh:
-                            search_view = TabBarView(device).navigateToSearch()
-                        profile_view = search_view.navigateToUsername(username, True)
-                        need_to_refresh = False
-                        if not profile_view:
-                            continue
 
-                        if not interact(
-                            storage,
-                            is_follow_limit_reached,
-                            username,
-                            interaction,
-                            device,
-                            self.session_state,
-                            current_job,
-                            on_interaction,
-                        ):
-                            return
-                        device.back()
+    filename = parameter_passed.split(" ")[0]
+    try:
+        amount_of_users = get_value(parameter_passed.split(" ")[1], None, 10)
+    except IndexError:
+        amount_of_users = 10
+        logger.warning(
+            f"You didn't passed how many users should be processed from the list! Default is {amount_of_users} users."
+        )
+    if path.isfile(filename):
+        with open(filename, "r", encoding="utf-8") as f:
+            usernames = [line.replace(" ", "") for line in f if line != "\n"]
+        len_usernames = len(usernames)
+        if len_usernames < amount_of_users:
+            amount_of_users = len_usernames
+        logger.info(
+            f"In {filename} there are {len_usernames} entries, {amount_of_users} users will be processed."
+        )
+        not_found = []
+        processed_users = 0
+        try:
+            for line, username_raw in enumerate(usernames, start=1):
+                username = username_raw.strip()
+                can_interact = False
+                if current_job == "unfollow-from-file":
+                    unfollowed = do_unfollow_from_list(
+                        device, username, on_following_list
+                    )
+                    on_following_list = True
+                    if unfollowed:
+                        storage.add_interacted_user(
+                            username, self.session_state.id, unfollowed=True
+                        )
+                        self.session_state.totalUnfollowed += 1
+                        limit_reached = self.session_state.check_limit(
+                            limit_type=self.session_state.Limit.UNFOLLOWS
+                        )
+                        processed_users += 1
+                    else:
+                        not_found.append(username_raw)
+                    if limit_reached:
+                        logger.info("Unfollows limit reached.")
+                        break
+                    if processed_users == amount_of_users:
+                        logger.info(
+                            f"{processed_users} users have been unfollowed, going to the next job."
+                        )
+                        break
                 else:
-                    logger.info("Line in file is blank, skip.")
-            remaining = f.readlines()
-        if self.args.delete_interacted_users:
-            with open(current_filename, "w", encoding="UTF-8") as f:
-                f.writelines(remaining)
+                    if storage.is_user_in_blacklist(username):
+                        logger.info(f"@{username} is in blacklist. Skip.")
+                    else:
+                        (
+                            interacted,
+                            interacted_when,
+                        ) = storage.check_user_was_interacted(username)
+                        if interacted:
+                            interact_after = timedelta(
+                                hours=float(self.args.can_reinteract_after)
+                            )
+                            can_reinteract = storage.can_be_reinteract(
+                                interacted_when, interact_after
+                            )
+                            logger.info(
+                                f"@{username}: already interacted on {interacted_when:%Y/%m/%d %H:%M:%S}. {'Interacting again now' if can_reinteract else 'Skip'}."
+                            )
+                            if can_reinteract:
+                                can_interact = True
+                        else:
+                            can_interact = True
+
+                    if not can_interact:
+                        continue
+                    if need_to_refresh:
+                        search_view = TabBarView(device).navigateToSearch()
+                    profile_view = search_view.navigateToUsername(username, True)
+                    need_to_refresh = False
+                    if not profile_view:
+                        not_found.append(username_raw)
+                        continue
+
+                    if not interact(
+                        storage,
+                        is_follow_limit_reached,
+                        username,
+                        interaction,
+                        device,
+                        self.session_state,
+                        current_job,
+                        on_interaction,
+                    ):
+                        return
+                    device.back()
+                    processed_users += 1
+                    if processed_users == amount_of_users:
+                        logger.info(
+                            f"{processed_users} users have been interracted, going to the next job."
+                        )
+                        return
+        finally:
+            if not_found:
+                with open(
+                    f"{os.path.splitext(filename)[0]}_not_found.txt",
+                    mode="a+",
+                    encoding="utf-8",
+                ) as f:
+                    f.writelines(not_found)
+            if self.args.delete_interacted_users and len_usernames != 0:
+                with atomic_write(filename, overwrite=True, encoding="utf-8") as f:
+                    f.writelines(usernames[line:])
     else:
-        logger.warning(f"File {current_filename} not found.")
+        logger.warning(
+            f"File {filename} not found. You have to specify the right relative path from this point: {os.getcwd()}"
+        )
         return
 
-    logger.info(f"Interact with users in {current_filename} completed.")
+    logger.info(f"Interact with users in {filename} completed.")
     device.back()
 
 
 def do_unfollow_from_list(device, username, on_following_list):
     if not on_following_list:
         ProfileView(device)._click_on_avatar()
-        if ProfileView(device).navigateToFollowing():
-            if UniversalActions(device).search_text(username):
-                return FollowingView(device).do_unfollow_from_list(username)
+        if ProfileView(device).navigateToFollowing() and UniversalActions(
+            device
+        ).search_text(username):
+            return FollowingView(device).do_unfollow_from_list(username)
     else:
         if username is not None:
             UniversalActions(device).search_text(username)
@@ -211,6 +277,7 @@ def do_unfollow_from_list(device, username, on_following_list):
 
 
 def handle_likers(
+    self,
     device,
     session_state,
     target,
@@ -222,23 +289,21 @@ def handle_likers(
     interaction,
     is_follow_limit_reached,
 ):
-    if current_job == "blogger-post-likers":
-        if not nav_to_post_likers(device, target, session_state.my_username):
-            return False
-    else:
-        if not nav_to_hashtag_or_place(device, target, current_job):
-            return False
-
+    if (
+        current_job == "blogger-post-likers"
+        and not nav_to_post_likers(device, target, session_state.my_username)
+        or current_job != "blogger-post-likers"
+        and not nav_to_hashtag_or_place(device, target, current_job)
+    ):
+        return False
     post_description = ""
     nr_same_post = 0
     nr_same_posts_max = 3
     while True:
-        flag, post_description, _, _ = PostsViewList(device)._check_if_last_post(
+        flag, post_description, _, _, _, _ = PostsViewList(device)._check_if_last_post(
             post_description, current_job
         )
-        likers_container_exists = PostsViewList(device)._find_likers_container()
-        has_one_liker_or_none = PostsViewList(device)._check_if_only_one_liker_or_none()
-
+        has_likers, number_of_likers = PostsViewList(device)._find_likers_container()
         if flag:
             nr_same_post += 1
             logger.info(f"Warning: {nr_same_post}/{nr_same_posts_max} repeated posts.")
@@ -251,7 +316,11 @@ def handle_likers(
         else:
             nr_same_post = 0
 
-        if likers_container_exists and not has_one_liker_or_none:
+        if (
+            has_likers
+            and profile_filter.is_num_likers_in_range(number_of_likers)
+            and number_of_likers != 1
+        ):
             PostsViewList(device).open_likers_container()
         else:
             PostsViewList(device).swipe_to_fit_posts(SwipeTo.NEXT_POST)
@@ -283,38 +352,49 @@ def handle_likers(
                         break
 
                     username = username_view.get_text()
-                    profile_interact = profile_filter.check_profile_from_list(
-                        device, item, username
-                    )
                     screen_iterated_likers.append(username)
                     posts_end_detector.notify_username_iterated(username)
-                    if not profile_interact:
-                        continue
-                    elif storage.is_user_in_blacklist(username):
+                    can_interact = False
+                    if storage.is_user_in_blacklist(username):
                         logger.info(f"@{username} is in blacklist. Skip.")
-                        continue
-                    elif storage.check_user_was_interacted(username):
-                        logger.info(f"@{username}: already interacted. Skip.")
-                        continue
                     else:
+                        (
+                            interacted,
+                            interacted_when,
+                        ) = storage.check_user_was_interacted(username)
+                        if interacted:
+                            interact_after = timedelta(
+                                hours=float(self.args.can_reinteract_after)
+                            )
+                            can_reinteract = storage.can_be_reinteract(
+                                interacted_when, interact_after
+                            )
+                            logger.info(
+                                f"@{username}: already interacted on {interacted_when:%Y/%m/%d %H:%M:%S}. {'Interacting again now' if can_reinteract else 'Skip'}."
+                            )
+                            if can_reinteract:
+                                can_interact = True
+                        else:
+                            can_interact = True
+
+                    if can_interact:
                         logger.info(
                             f"@{username}: interact",
                             extra={"color": f"{Fore.YELLOW}"},
                         )
                         element_opened = username_view.click_retry()
 
-                        if element_opened:
-                            if not interact(
-                                storage,
-                                is_follow_limit_reached,
-                                username,
-                                interaction,
-                                device,
-                                session_state,
-                                current_job,
-                                on_interaction,
-                            ):
-                                return
+                        if element_opened and not interact(
+                            storage,
+                            is_follow_limit_reached,
+                            username,
+                            interaction,
+                            device,
+                            session_state,
+                            current_job,
+                            on_interaction,
+                        ):
+                            return
                     if element_opened:
                         opened = True
                         logger.info("Back to likers list.")
@@ -345,22 +425,20 @@ def handle_likers(
                 PostsViewList(device).swipe_to_fit_posts(SwipeTo.NEXT_POST)
                 break
             if posts_end_detector.is_fling_limit_reached():
-                prev_screen_iterated_likers.clear()
-                prev_screen_iterated_likers += screen_iterated_likers
                 logger.info(
                     "Reached fling limit. Fling to see other likers.",
                     extra={"color": f"{Fore.GREEN}"},
                 )
                 likes_list_view.fling(Direction.DOWN)
             else:
-                prev_screen_iterated_likers.clear()
-                prev_screen_iterated_likers += screen_iterated_likers
                 logger.info(
                     "Scroll to see other likers.",
                     extra={"color": f"{Fore.GREEN}"},
                 )
                 likes_list_view.scroll(Direction.DOWN)
 
+            prev_screen_iterated_likers.clear()
+            prev_screen_iterated_likers += screen_iterated_likers
             if posts_end_detector.is_the_end():
                 device.back()
                 PostsViewList(device).swipe_to_fit_posts(SwipeTo.NEXT_POST)
@@ -383,13 +461,24 @@ def handle_posts(
     target,
     current_job,
     storage,
+    profile_filter,
     on_interaction,
     interaction,
     is_follow_limit_reached,
     interact_percentage,
     scraping_file,
 ):
+    skipped_posts_limit = get_value(
+        self.args.skipped_posts_limit,
+        "Skipped post limit: {}",
+        5,
+    )
     if current_job == "feed":
+        if scraping_file:
+            logger.warning(
+                "Scraping and interacting with own feed doesn't make any sense. Skip."
+            )
+            return
         nav_to_feed(device)
         count_feed_limit = get_value(
             self.args.feed,
@@ -398,19 +487,36 @@ def handle_posts(
         )
         count = 0
         PostsViewList(device)._refresh_feed()
-    else:
-        if not nav_to_hashtag_or_place(device, target, current_job):
-            return
+    elif not nav_to_hashtag_or_place(device, target, current_job):
+        return
 
     post_description = ""
+    likes_failed = 0
     nr_same_post = 0
     nr_same_posts_max = 3
+    nr_consecutive_already_interacted = 0
+    already_liked_count = 0
+    already_liked_count_limit = 20
+    post_view_list = PostsViewList(device)
+    opened_post_view = OpenedPostView(device)
     while True:
-        flag, post_description, username, ad = PostsViewList(
-            device
-        )._check_if_last_post(post_description, current_job)
-        if not ad:
-            if flag:
+        (
+            is_same_post,
+            post_description,
+            username,
+            is_ad,
+            is_hashtag,
+            has_tags,
+        ) = post_view_list._check_if_last_post(post_description, current_job)
+        has_likers, number_of_likers = post_view_list._find_likers_container()
+        already_liked, _ = opened_post_view._is_post_liked()
+        if not (is_ad or is_hashtag):
+            if already_liked_count == already_liked_count_limit:
+                logger.info(
+                    f"Limit of {already_liked_count_limit} already liked posts limit reached, finish."
+                )
+                break
+            if is_same_post:
                 nr_same_post += 1
                 logger.info(
                     f"Warning: {nr_same_post}/{nr_same_posts_max} repeated posts."
@@ -422,50 +528,99 @@ def handle_posts(
                     break
             else:
                 nr_same_post = 0
-            if random_choice(interact_percentage):
+            if already_liked:
+                logger.info(
+                    "Post already liked, SKIP.", extra={"color": f"{Fore.CYAN}"}
+                )
+                already_liked_count += 1
+            elif random_choice(interact_percentage):
+                can_interact = False
                 if storage.is_user_in_blacklist(username):
                     logger.info(f"@{username} is in blacklist. Skip.")
-                elif (
-                    storage.check_user_was_interacted(username)
-                    and current_job != "feed"
-                ):
-                    logger.info(f"@{username}: already interacted. Skip.")
-                elif (
-                    storage.check_user_was_interacted_recently(username)
-                    and current_job != "feed"
-                ):
-                    logger.info(
-                        f"@{username}: already interacted in the last week. Skip."
-                    )
-                # for now i put it back wrong as it was (and how it's in the other plugins..) will change that with the database
                 else:
+                    likes_in_range = profile_filter.is_num_likers_in_range(
+                        number_of_likers
+                    )
+                    if current_job != "feed":
+                        interacted, interacted_when = storage.check_user_was_interacted(
+                            username
+                        )
+                        if interacted:
+                            interact_after = timedelta(
+                                hours=float(self.args.can_reinteract_after)
+                            )
+                            can_reinteract = storage.can_be_reinteract(
+                                interacted_when, interact_after
+                            )
+                            logger.info(
+                                f"@{username}: already interacted on {interacted_when:%Y/%m/%d %H:%M:%S}. {'Interacting again now' if can_reinteract else 'Skip'}."
+                            )
+                            if can_reinteract:
+                                can_interact = True
+                                nr_consecutive_already_interacted = 0
+                            else:
+                                nr_consecutive_already_interacted += 1
+                        else:
+                            can_interact = True
+                            nr_consecutive_already_interacted = 0
+                    else:
+                        can_interact = True
+
+                if nr_consecutive_already_interacted == skipped_posts_limit:
+                    logger.info(
+                        f"Reached the limit of already interacted {skipped_posts_limit}. Goin to the next source/job!"
+                    )
+                    break
+                if can_interact and (likes_in_range or not has_likers):
                     logger.info(
                         f"@{username}: interact", extra={"color": f"{Fore.YELLOW}"}
                     )
                     if scraping_file is None:
-                        PostsViewList(device)._like_in_post_view(LikeMode.DOUBLE_CLICK)
-                        UniversalActions.detect_block(device)
-                        if not PostsViewList(device)._check_if_liked():
-                            PostsViewList(device)._like_in_post_view(
-                                LikeMode.SINGLE_CLICK
-                            )
-                            UniversalActions.detect_block(device)
-                        session_state.totalLikes += 1
-                        if current_job == "feed":
-                            count += 1
-                            logger.info(
-                                f"Interacted feed bloggers: {count}/{count_feed_limit}"
-                            )
-                            if count >= count_feed_limit:
-                                logger.info(
-                                    f"Interacted {count} bloggers in feed, finish."
-                                )
-                                TabBarView(device).navigateToProfile()
-                                return
-                    if current_job != "feed":
-                        if PostsViewList(device)._post_owner(
-                            current_job, Owner.OPEN, username
+                        opened_post_view.start_video()
+                        if not session_state.check_limit(
+                            limit_type=session_state.Limit.LIKES, output=True
                         ):
+                            if has_tags:
+                                post_view_list._like_in_post_view(LikeMode.SINGLE_CLICK)
+                            else:
+                                post_view_list._like_in_post_view(LikeMode.DOUBLE_CLICK)
+                            UniversalActions.detect_block(device)
+                            liked = post_view_list._check_if_liked()
+                            if not liked:
+                                post_view_list._like_in_post_view(LikeMode.SINGLE_CLICK)
+                                UniversalActions.detect_block(device)
+                                liked = post_view_list._check_if_liked()
+                            if liked:
+                                session_state.totalLikes += 1
+                                if current_job == "feed":
+                                    count += 1
+                                    logger.info(
+                                        f"Interacted feed bloggers: {count}/{count_feed_limit}"
+                                    )
+                                    likes_limit = self.session_state.check_limit(
+                                        limit_type=self.session_state.Limit.LIKES
+                                    )
+                                    success_limit = self.session_state.check_limit(
+                                        limit_type=self.session_state.Limit.SUCCESS
+                                    )
+                                    total_limit = self.session_state.check_limit(
+                                        limit_type=self.session_state.Limit.TOTAL
+                                    )
+                                    if likes_limit or success_limit or total_limit:
+                                        logger.info("Limit reached, finish.")
+                                        break
+                                    if count >= count_feed_limit:
+                                        logger.info(
+                                            f"Interacted {count} bloggers in feed, finish."
+                                        )
+                                        break
+                            else:
+                                likes_failed += 1
+                    if current_job != "feed":
+                        opened, _, _ = post_view_list._post_owner(
+                            current_job, Owner.OPEN, username
+                        )
+                        if opened:
                             if not interact(
                                 storage,
                                 is_follow_limit_reached,
@@ -476,11 +631,18 @@ def handle_posts(
                                 current_job,
                                 on_interaction,
                             ):
-                                return
+                                break
                             device.back()
-
-        PostsViewList(device).swipe_to_fit_posts(SwipeTo.HALF_PHOTO)
-        PostsViewList(device).swipe_to_fit_posts(SwipeTo.NEXT_POST)
+            else:
+                logger.info(
+                    f"Skipped because your interact % is {interact_percentage}/100 and {username}'s post was unlucky!"
+                )
+        if likes_failed == 10:
+            logger.warning("You failed to do 10 likes! Soft-ban?!")
+            return
+        post_view_list.swipe_to_fit_posts(SwipeTo.HALF_PHOTO)
+        post_view_list.swipe_to_fit_posts(SwipeTo.NEXT_POST)
+    TabBarView(device).navigateToProfile()
 
 
 def handle_followers(
@@ -595,17 +757,31 @@ def iterate_over_followers(
                 screen_iterated_followers.append(username)
                 scroll_end_detector.notify_username_iterated(username)
 
+                can_interact = False
                 if storage.is_user_in_blacklist(username):
                     logger.info(f"@{username} is in blacklist. Skip.")
-                elif not is_myself and storage.check_user_was_interacted(username):
-                    logger.info(f"@{username}: already interacted. Skip.")
-                    screen_skipped_followers_count += 1
-                elif is_myself and storage.check_user_was_interacted_recently(username):
-                    logger.info(
-                        f"@{username}: already interacted in the last week. Skip."
-                    )
-                    screen_skipped_followers_count += 1
                 else:
+                    interacted, interacted_when = storage.check_user_was_interacted(
+                        username
+                    )
+                    if interacted:
+                        interact_after = timedelta(
+                            hours=float(self.args.can_reinteract_after)
+                        )
+                        can_reinteract = storage.can_be_reinteract(
+                            interacted_when, interact_after
+                        )
+                        logger.info(
+                            f"@{username}: already interacted on {interacted_when:%Y/%m/%d %H:%M:%S}. {'Interacting again now' if can_reinteract else 'Skip'}."
+                        )
+                        if can_reinteract:
+                            can_interact = True
+                        else:
+                            screen_skipped_followers_count += 1
+                    else:
+                        can_interact = True
+
+                if can_interact:
                     logger.info(
                         f"@{username}: interact", extra={"color": f"{Fore.YELLOW}"}
                     )
@@ -672,10 +848,13 @@ def iterate_over_followers(
                         descriptionMatches=case_insensitive_re("Retry"),
                     )
                     if retry_button.exists():
-                        logger.info('Press "Load" button and wait few seconds.')
-                        retry_button.click_retry()
-                        random_sleep(5, 10, modulable=False)
-                        pressed_retry = True
+                        random_sleep()
+                        """It exist but can disappear without pressing on it"""
+                        if retry_button.exists():
+                            logger.info('Press "Load" button and wait few seconds.')
+                            retry_button.click_retry()
+                            random_sleep(5, 10, modulable=False)
+                            pressed_retry = True
 
                 if need_swipe and not pressed_retry:
                     scroll_end_detector.notify_skipped_all()
