@@ -1,16 +1,17 @@
 import logging
 from functools import partial
 from random import seed
+
 from colorama import Style
+
 from GramAddict.core.decorators import run_safely
-from GramAddict.core.filter import Filter
+from GramAddict.core.handle_sources import handle_posts
 from GramAddict.core.interaction import (
     interact_with_user,
     is_follow_limit_reached_for_source,
 )
-from GramAddict.core.handle_sources import handle_posts
 from GramAddict.core.plugin_loader import Plugin
-from GramAddict.core.utils import get_value, sample_sources, init_on_things
+from GramAddict.core.utils import get_value, init_on_things, sample_sources
 
 logger = logging.getLogger(__name__)
 
@@ -45,7 +46,7 @@ class InteractPlacePosts(Plugin):
             },
         ]
 
-    def run(self, device, configs, storage, sessions, plugin):
+    def run(self, device, configs, storage, sessions, profile_filter, plugin):
         class State:
             def __init__(self):
                 pass
@@ -56,7 +57,6 @@ class InteractPlacePosts(Plugin):
         self.sessions = sessions
         self.session_state = sessions[-1]
         self.args = configs.args
-        profile_filter = Filter(storage)
         self.current_mode = plugin
 
         # Handle sources
@@ -70,9 +70,12 @@ class InteractPlacePosts(Plugin):
         ]
 
         for source in sample_sources(sources, self.args.truncate_sources):
-            limit_reached = self.session_state.check_limit(
-                self.args, limit_type=self.session_state.Limit.ALL
-            )
+            (
+                active_limits_reached,
+                _,
+                actions_limit_reached,
+            ) = self.session_state.check_limit(limit_type=self.session_state.Limit.ALL)
+            limit_reached = active_limits_reached or actions_limit_reached
 
             self.state = State()
             logger.info(f"Handle {source}", extra={"color": f"{Style.BRIGHT}"})
@@ -119,7 +122,7 @@ class InteractPlacePosts(Plugin):
             if limit_reached:
                 logger.info("Ending session.")
                 self.session_state.check_limit(
-                    self.args, limit_type=self.session_state.Limit.ALL, output=True
+                    limit_type=self.session_state.Limit.ALL, output=True
                 )
                 break
 
