@@ -1,21 +1,21 @@
-from GramAddict.core.views import AccountView, ProfileView
-from colorama import Fore, Style
-from GramAddict.core.plugin_loader import Plugin
-import datetime
 import json
 import logging
-import os
+import sys
+from datetime import datetime, timedelta
 from textwrap import dedent
+
 import requests
 import yaml
-import sys
+from colorama import Fore, Style
+
+from GramAddict.core.plugin_loader import Plugin
 
 logger = logging.getLogger(__name__)
 
 try:
     import pandas as pd
 except ImportError:
-    logger.error(
+    logger.warning(
         "If you want to use telegram_reports, please type in console: 'pip3 install gramaddict[telegram-reports]'"
     )
 
@@ -35,7 +35,7 @@ class TelegramReports(Plugin):
             }
         ]
 
-    def run(self, device, config, storage, sessions, plugin):
+    def run(self, config, plugin, followers_now, following_now, time_left):
         username = config.args.username
         modulename = "pandas"
         if modulename not in sys.modules:
@@ -77,6 +77,7 @@ class TelegramReports(Plugin):
             start = session["start_time"]
             finish = session["finish_time"]
             followed = session.get("total_followed", 0)
+            unfollowed = session.get("total_unfollowed", 0)
             likes = session.get("total_likes", 0)
             watched = session.get("total_watched", 0)
             comments = session.get("total_comments", 0)
@@ -90,6 +91,7 @@ class TelegramReports(Plugin):
                     likes,
                     watched,
                     followed,
+                    unfollowed,
                     comments,
                     pm_sent,
                     followers,
@@ -105,6 +107,7 @@ class TelegramReports(Plugin):
                 "likes",
                 "watched",
                 "followed",
+                "unfollowed",
                 "comments",
                 "pm_sent",
                 "followers",
@@ -116,19 +119,18 @@ class TelegramReports(Plugin):
             df["start"], errors="coerce"
         )
         df["duration"] = df["duration"].dt.total_seconds() / 60
-        modTimesinceEpoc = os.path.getmtime(f"accounts/{username}/sessions.json")
-        maxDate = datetime.datetime.fromtimestamp(modTimesinceEpoc)
-        timeSince = datetime.datetime.now() - maxDate
-        if timeSince.seconds < 60:
-            dateString = "Last session about a minute ago."
+
+        if time_left is not None:
+            timeString = f'Next session will start at: {(datetime.now()+ timedelta(seconds=time_left)).strftime("%H:%M:%S (%Y/%m/%d)")}.'
         else:
-            dateString = f"Last session {int(timeSince.seconds / 60)} minutes ago."
+            timeString = "There is no new session planned!"
 
         dailySummary = df.groupby(by="date").agg(
             {
                 "likes": "sum",
                 "watched": "sum",
                 "followed": "sum",
+                "unfollowed": "sum",
                 "comments": "sum",
                 "pm_sent": "sum",
                 "followers": "max",
@@ -142,7 +144,7 @@ class TelegramReports(Plugin):
             ) - dailySummary["followers"].astype(int).shift(1)
         else:
             logger.info(
-                "First day of botting eh? Stats for the first day are meh because we don't have enought data to track how many followers you earned today from the bot activity."
+                "First day of botting eh? Stats for the first day are meh because we don't have enough data to track how many followers you earned today from the bot activity."
             )
             dailySummary["followers_gained"] = dailySummary["followers"].astype(int)
         dailySummary.dropna(inplace=True)
@@ -163,18 +165,12 @@ class TelegramReports(Plugin):
         def undentString(string):
             return dedent(string[1:])[:-1]
 
-        ProfileView(device)._click_on_avatar()
-        AccountView(device).refresh_account()
-        (
-            _,
-            _,
-            followers_now,
-            following_now,
-        ) = ProfileView(device).getProfileInfo()
-        followers_before = int(dailySummary["followers"].iloc[-1])
-        following_before = int(dailySummary["following"].iloc[-1])
+        followers_before = int(df["followers"].iloc[-1])
+        following_before = int(df["following"].iloc[-1])
         statString = f"""
-                *Starts for {username}* after last activity:
+                *Stats for {username}*:
+
+                *✨Overview after last activity*
                 • {followers_now} followers ({followers_now - followers_before:+})
                 • {following_now} following ({following_now - following_before:+})
 
@@ -182,14 +178,16 @@ class TelegramReports(Plugin):
                 • {str(df["duration"].iloc[-1].astype(int))} minutes of botting
                 • {str(df["likes"].iloc[-1])} likes
                 • {str(df["followed"].iloc[-1])} follows
+                • {str(df["unfollowed"].iloc[-1])} unfollows
                 • {str(df["watched"].iloc[-1])} stories watched
                 • {str(df["comments"].iloc[-1])} comments done
                 • {str(df["pm_sent"].iloc[-1])} PM sent
 
-                *📅 Today total actions*
+                *📅 Todays total actions*
                 • {str(dailySummary["duration"].iloc[-1])} minutes of botting
                 • {str(dailySummary["likes"].iloc[-1])} likes
                 • {str(dailySummary["followed"].iloc[-1])} follows
+                • {str(dailySummary["unfollowed"].iloc[-1])} unfollows
                 • {str(dailySummary["watched"].iloc[-1])} stories watched
                 • {str(dailySummary["comments"].iloc[-1])} comments done
                 • {str(dailySummary["pm_sent"].iloc[-1])} PM sent
@@ -204,13 +202,14 @@ class TelegramReports(Plugin):
                 • {str(round(dailySummary["followers_gained"].tail(7).mean(), 1))} followers / day
                 • {str(int(dailySummary["likes"].tail(7).mean()))} likes
                 • {str(int(dailySummary["followed"].tail(7).mean()))} follows
+                • {str(int(dailySummary["unfollowed"].tail(7).mean()))} unfollows
                 • {str(int(dailySummary["watched"].tail(7).mean()))} stories watched
                 • {str(int(dailySummary["comments"].tail(7).mean()))} comments done
                 • {str(int(dailySummary["pm_sent"].tail(7).mean()))} PM sent
                 • {str(int(dailySummary["duration"].tail(7).mean()))} minutes of botting
             """
         try:
-            r = telegram_bot_sendtext(f"{undentString(statString)}\n\n{dateString}")
+            r = telegram_bot_sendtext(f"{undentString(statString)}\n\n{timeString}")
             if r.get("ok"):
                 logger.info(
                     "Telegram message sent successfully.",
